<?php

namespace Drupal\commerce_product_quantity\EventSubscriber;

use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_cart\Event\CartEntityAddEvent;
use Drupal\commerce_cart\Event\CartEvents;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Product Type Cart Event Subscriber.
 *
 * Manages Product type limit within the cart.
 */
class ProductTypeCartEventSubscriber implements EventSubscriberInterface {
  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;
  /**
   * The cart manager.
   *
   * @var \Drupal\commerce_cart\CartManagerInterface
   */
  protected $cartManager;

  public function __construct(MessengerInterface $messenger, CartManagerInterface $cart_manager) {
    $this->messenger = $messenger;
    $this->cartManager = $cart_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [CartEvents::CART_ENTITY_ADD => [['addToCart', -100]]];
  }

  /**
   * This method will return array of product and product_type.
   *
   * If that clicked product or product_type is available in
   * form configurations.
   */
  public function search($array, $key, $value) {
    $results = [];
    if (is_array($array)) {
      $results = [];
      $i = 0;
      foreach ($array as $key1 => $subarray) {
        if ($subarray[$key] == $value) {
          $results = $subarray;
          break;
        }
        $i++;
      }
    }
    return $results;
  }

  /**
   * Get stored quantity and product ID from config.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface $product_variation
   */
  public function addToCart(CartEntityAddEvent $event) {
    /*
     * Get quantity for particular product_type
     */
    $product_type_config = \Drupal::config('commerce_product_type_quantity.settings');
    $product_type_count = $product_type_config->get('product_quantity_count');
    $product_type_list = [];
    for ($i = 0; $i < $product_type_count; $i++) {
      $product_type_list[$i]['product_type'] = $product_type_config->get('product_' . $i);
      $product_type_list[$i]['quantity'] = $product_type_config->get('quantity_' . $i);
    }
    /*
     * Get quantity for particular product.
     */
    $product_config = \Drupal::config('commerce_product_quantity.settings');
    $product_count = $product_config->get('product_quantity_count');
    $product_name = [];
    $products_list = [];
    for ($i = 0; $i < $product_count; $i++) {
      $products_list[$i]['product'] = $product_config->get('product_' . $i);
      $id = $products_list[$i]['product'];
      $int = (int) $id;
      $prod_id = explode(' ', $int);
      $product = \Drupal::entityTypeManager()->getStorage('commerce_product')->loadMultiple($prod_id);
      foreach ($product as $products) {
        $product_name[$i] = $products->title->value;
      }
      $products_list[$i]['product_name'] = $product_name[$i];
      $products_list[$i]['quantity'] = $product_config->get('quantity_' . $i);
    }
    $titles = [];
    $cart = $event->getCart();
    $cart_items = $cart->getItems();
    $i = 0;
    foreach ($cart_items as $cart_item) {
      $quantity = $cart_item->getQuantity();
      $temp_product_variation = $cart_item->getPurchasedEntity();
      $product_id = $temp_product_variation->product_id->getString();
      $p = explode(' ', $product_id);
      $product = \Drupal::entityTypeManager()->getStorage('commerce_product')->loadMultiple($p);
      foreach ($product as $products) {
        $titles['name'] = $products->title->value;
        $titles[$products->product_id->value] = $products->bundle();
      }
      $selected_product = $titles['name'];
      $selected_product_type = $products->bundle();
      if (isset($quantity)) {
        $quantity = (int) $quantity;
        if (count($this->search($products_list, 'product_name', $selected_product)) == 3) {
          $product_data = $this->search($products_list, 'product_name', $selected_product);
          $product_quant = $product_data['quantity'];
          if ($quantity > $product_quant) {
            $cart_item->setQuantity($product_quant);
            $this->messenger->deleteAll();
            $this->messenger->addMessage(t('You have reached max quantity for this product'), 'error');
          }
          else {
            $cart_item->setQuantity($quantity);
          }
        }
        elseif (count($this->search($product_type_list, 'product_type', $selected_product_type)) == 2) {
          $product_type_data = $this->search($product_type_list, 'product_type', $selected_product_type);
          $product_quant = $product_type_data['quantity'];
          if ($quantity > $product_quant) {
            $cart_item->setQuantity($product_quant);
            $this->messenger->deleteAll();
            $this->messenger->addMessage(t('You have reached max quantity for this product type'), 'error');
          }
          else {
            $cart_item->setQuantity($quantity);
          }
        }
      }
    }
    $cart->save();
  }

}
