<?php

namespace Drupal\commerce_product_quantity\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure commerce_product_quantity settings for this site.
 */
class ProductForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'commerce_product_quantity_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['commerce_product_quantity.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('commerce_product_quantity.settings');
    /*
     * Fetch All Products from the table.
     */
    $result = \Drupal::entityQuery('commerce_product');
    $result->accessCheck(FALSE);
    $temp_products = $result->execute();  // Store results in a temporary variable

    $total_product = count($temp_products);


    if ($total_product === 0) {
      $form['product_quantity_fieldset'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Products With Total Allowed Quantity'),
        '#attributes' => ['id' => 'product-fieldset-wrapper'],
        '#markup' => '<div class="no-products-message">Please add products to your store.</div>',
    ];
    return $form;
  }

    $titles = ['none' => "- Select Products -"];

    foreach ($temp_products as $product_id) {
      $entity_manager = \Drupal::entityTypeManager();
      $product = $entity_manager->getStorage('commerce_product')->load($product_id);
      if (isset($product)) {
        $titles[$product->product_id->value] = $product->get('title')->value;
      }
    }
    /*
     * $config_count will get number of 'product_quantity_count'.
     *
     * From 'commerce_product_quantity.settings' configuration
     */
    $config_count = $config->get('product_quantity_count');

    $form['#tree'] = TRUE;
    $form['product_quantity_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Products With Total Allowed Quantity'),
      '#attributes' => ['id' => 'product-fieldset-wrapper'],
    ];

    /*
     * It will fetch the index of removed item from the.
     *
     * Configuration and store this to $i_remove variable.
     */
    $i_remove = $config->get('index');

    for ($i = 0; $i < $config_count; $i++) {

      if ($i_remove === $i) {
        $i++;
      }

      $form['product_quantity_fieldset'][$i] = [
        '#type' => 'fieldset',
        '#title' => '',
        '#attributes' => ['id' => 'product-fieldset-wrapper-child'],
      ];

      $form['product_quantity_fieldset'][$i]['product_quantity_fieldset']['product'] = [
        '#type' => 'select',
        '#title' => $this->t('Select Product'),
        '#options' => $titles,
        '#default_value' => $config->get('product_' . $i),
      ];

      $form['product_quantity_fieldset'][$i]['product_quantity_fieldset']['quantity'] = [
        '#type' => 'number',
        '#title' => $this->t('Select Quantity'),
        '#default_value' => $config->get('quantity_' . $i),
        '#attributes' => ['min' => '1'],
      ];

      $form['product_quantity_fieldset'][$i]['actions'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove one'),
        '#name' => $i,
        '#submit' => ['::removeOne'],
        '#ajax' => [
          'callback' => '::removeCallback',
          'wrapper' => 'product-fieldset-wrapper',
        ],
      ];
    }
    $form['actions'] = [
      '#type' => 'actions',
    ];
    if ($config_count < $total_product) {
      $form['product_quantity_fieldset']['actions']['add_name'] = [
        '#type' => 'submit',
        '#value' => $this->t('Add Product'),
        '#submit' => ['::addOne'],
        '#ajax' => [
          'callback' => '::addmoreCallback',
          'wrapper' => 'product-fieldset-wrapper',
        ],
      ];
    }
    $form_state->setCached(FALSE);
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];
    
    return $form;
  }

  /**
   * This function will add product form.
   *
   * For storing quantityof selected product.
   */
  public function addOne(array &$form, FormStateInterface $form_state) {
    $field_count = $form_state->get('num_names');
    $config = \Drupal::configFactory()->getEditable('commerce_product_quantity.settings');
    $field_count = $config->get('product_quantity_count');
    $add_button = $field_count + 1;
    $field_count = $config->set('product_quantity_count', $add_button)->save();
    $form_state->set('num_names', $add_button);
    $form_state->setRebuild();
  }

  /**
   * This function will remove product from existing form.
   */
  public function removeOne(array &$form, FormStateInterface $form_state) {
    $name = $form_state->getTriggeringElement()['#name'];
    $config = \Drupal::configFactory()->getEditable('commerce_product_quantity.settings');
    unset($form['product_quantity_fieldset'][$name]);
    $field_count = $config->get('product_quantity_count');
    $add_button = $field_count - 1;
    $config->set('product_quantity_count', $add_button)->save();
    $config->set('index', $name)->save();
    $form_state->setRebuild();
  }

  /**
   * This function gets call on ajax. after submitting ::addOne button in form.
   */
  public function addmoreCallback(array &$form, FormStateInterface $form_state) {
    return $form['product_quantity_fieldset'];
  }

  /**
   * This function gets call on ajax.
   *
   * After submitting ::removeOne button of specific existing product.
   */
  public function removeCallback(array &$form, FormStateInterface $form_state) {
    return $form['product_quantity_fieldset'];
  }

  /**
   * This function will validate form.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $product_quantities = $form_state->getValue('product_quantity_fieldset');

    $selected_product = $product_quantities[0]['product_quantity_fieldset']['product'];

    if ($selected_product === 'none') {
      // Add an error to the form.
      $form_state->setErrorByName('product_quantity_fieldset][0][product_quantity_fieldset][product', $this->t('Please select a product.'));
    }

  }

  /**
   * Submit form will save the change.
   *
   * In the form configuration after performing remove or add action.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    if ($form_state->hasAnyErrors()) {
      return;
    }

    $config = $this->config('commerce_product_quantity.settings');
    $config_count = $config->get('product_quantity_count');
    $config->delete();
    $config_count = $config->set('product_quantity_count', $config_count)->save();
    $product_quantities = $form_state->getValue('product_quantity_fieldset');

    $i = 0;
    foreach ($product_quantities as $value) {
      $product_quantity = $value['product_quantity_fieldset'];
      if (!empty($product_quantity)) {
        foreach ($product_quantity as $key => $value2) {
          switch ($key) {
            case 'product':
              $config->set('product_' . $i, $value2)->save();
              break;

            case 'quantity':
              $config->set('quantity_' . $i, $value2)->save();
              break;
          }
        }
      }
      $i++;

      $messenger = \Drupal::messenger();
      $messenger->addMessage('Product Quantity Limit Updated Successfully.');
    }
  }

}
